#!/usr/bin/env python3

"""
BRS-XSS Locales Module

Localization files directory.

Company: EasyProTech LLC (www.easypro.tech)
Dev: Brabus
Created: Sat 02 Aug 2025 11:30:00 MSK
Telegram: https://t.me/EasyProTech
"""

# This file makes the locales directory a Python package
# Contains translation files for different languages:
# - en: English (base language)
# - ru: Russian (full translation)
# - zh, es, de, fr, ja: Planned for future releases