#!/usr/bin/env python3

# Project: BRS-XSS (XSS Detection Suite)
# Company: EasyProTech LLC (www.easypro.tech)
# Dev: Brabus
# Date: Sun 10 Aug 2025 21:38:09 MSK
# Status: Modified
# Telegram: https://t.me/EasyProTech

"""
Legacy setup.py for backward compatibility
Main configuration is located in pyproject.toml
"""

from setuptools import setup

if __name__ == "__main__":
    setup()